@@ -1,6 +1,6 @@
 ## simulate the process model
 
-rprocess.internal <- function (object, xstart, times, params, offset = 0, .getnativesymbolinfo = TRUE, ...) {
+rprocess.internal <- function (object, xstart, times, params, offset = 0L, .getnativesymbolinfo = TRUE, ...) {
     storage.mode(xstart) <- "double"
     storage.mode(params) <- "double"
     pompLoad(object)
@@ -12,7 +12,7 @@ rprocess.internal <- function (object, xstart, times, params, offset = 0, .getna
 setMethod(
     "rprocess",
     signature=signature(object="pomp"),
-    definition=function (object, xstart, times, params, offset = 0, ...) {
+    definition=function (object, xstart, times, params, offset = 0L, ...) {
         rprocess.internal(object=object,xstart=xstart,times=times,params=params,offset=offset,...)
     }
 )
