@@ -104,35 +104,31 @@ dat <- '"time";"y1";"y2"
 '
 
 pomp(
-     data=read.csv2(text=dat),
-     times="time",
-     t0=0,
-     params=c(x1.0=0,x2.0=0,s1=1,s2=3,tau=1), # parameters at which data were generated
-     rprocess = onestep.sim(
-       step.fun = function(x, t, params, delta.t, ...) {
-         c(
-           x1=rnorm(n=1,mean=x['x1'],sd=params['s1']*delta.t),
-           x2=rnorm(n=1,mean=x['x2'],sd=params['s2']*delta.t)
-           )
-       }
-       ),
-     dprocess = onestep.dens(
-       dens.fun = function (x1, t1, x2, t2, params, ...) {
-         sum(
-             dnorm(
-                   x=x2[c('x1','x2')],
-                   mean=x1[c('x1','x2')],
-                   sd=params[c('s1','s2')]*(t2-t1),
-                   log=TRUE
-                   ),
-             na.rm=TRUE
-             )
-       }
-       ),
-     measurement.model=list(
-       y1 ~ norm(mean=x1,sd=tau),
-       y2 ~ norm(mean=x2,sd=tau)
-       )
-     ) -> rw2
+  data=read.csv2(text=dat),
+  times="time",
+  t0=0,
+  params=c(x1.0=0,x2.0=0,s1=1,s2=3,tau=1), # parameters at which data were generated
+  rprocess = onestep.sim(
+    Csnippet("
+        x1 = rnorm(x1,s1*sqrt(dt));
+        x2 = rnorm(x2,s2*sqrt(dt));"
+    )
+  ),
+  dprocess = Csnippet("
+        double sdt = sqrt(t_2 - t_1);
+        loglik = dnorm(x1_2,x1_1,s1*sdt,1)+
+          dnorm(x2_2,x2_1,s2*sdt,1);"
+  ),
+  rmeasure=Csnippet("
+        y1 = rnorm(x1,tau);
+        y2 = rnorm(x1,tau);"
+  ),
+  dmeasure=Csnippet("
+        lik = dnorm(y1,x1,tau,1)+dnorm(y2,x2,tau,1);
+        lik = (give_log) ? lik : exp(lik);"
+  ),
+  statenames=c("x1","x2"),
+  paramnames=c("s1","s2","tau")
+) -> rw2
 
 c("rw2")
