@@ -6,7 +6,7 @@ static const R_CallMethodDef callMethods[] = {
   {"periodic_bspline_basis", (DL_FUNC) &periodic_bspline_basis, 5},
   {"systematic_resampling", (DL_FUNC) &systematic_resampling, 1},
   {"euler_model_simulator", (DL_FUNC) &euler_model_simulator, 11},
-  {"euler_model_density", (DL_FUNC) &euler_model_density, 9},
+  {"onestep_density", (DL_FUNC) &onestep_density, 9},
   {"lookup_in_table", (DL_FUNC) &lookup_in_table, 3},
   {"load_stack_incr", (DL_FUNC) &load_stack_incr, 1},
   {"load_stack_decr", (DL_FUNC) &load_stack_decr, 1},
@@ -22,7 +22,7 @@ static const R_CallMethodDef callMethods[] = {
   {"pomp_desolve_setup", (DL_FUNC) &pomp_desolve_setup, 4},
   {"pomp_desolve_takedown", (DL_FUNC) &pomp_desolve_takedown, 0},
   {"apply_probe_data", (DL_FUNC) &apply_probe_data, 2},
-  {"apply_probe_sim", (DL_FUNC) &apply_probe_sim, 6},
+  {"apply_probe_sim", (DL_FUNC) &apply_probe_sim, 7},
   {"probe_marginal_setup", (DL_FUNC) &probe_marginal_setup, 3},
   {"probe_marginal_solve", (DL_FUNC) &probe_marginal_solve, 3},
   {"probe_acf", (DL_FUNC) &probe_acf, 3},
@@ -48,6 +48,18 @@ void R_init_pomp (DllInfo *info) {
   R_RegisterCCallable("pomp","get_pomp_userdata",(DL_FUNC) &get_pomp_userdata);
   R_RegisterCCallable("pomp","get_pomp_userdata_int",(DL_FUNC) &get_pomp_userdata_int);
   R_RegisterCCallable("pomp","get_pomp_userdata_double",(DL_FUNC) &get_pomp_userdata_double);
+  R_RegisterCCallable("pomp","pomp_fun_handler",(DL_FUNC) &pomp_fun_handler);
+  R_RegisterCCallable("pomp","load_stack_incr",(DL_FUNC) &load_stack_incr);
+  R_RegisterCCallable("pomp","load_stack_decr",(DL_FUNC) &load_stack_decr);
+  R_RegisterCCallable("pomp","set_pomp_userdata",(DL_FUNC) &set_pomp_userdata);
+  R_RegisterCCallable("pomp","unset_pomp_userdata",(DL_FUNC) &unset_pomp_userdata);
+  R_RegisterCCallable("pomp","make_covariate_table",(DL_FUNC) &make_covariate_table);
+  R_RegisterCCallable("pomp","lookup_in_table",(DL_FUNC) &lookup_in_table);
+  R_RegisterCCallable("pomp","table_lookup",(DL_FUNC) &table_lookup);
+  R_RegisterCCallable("pomp","apply_probe_data",(DL_FUNC) &apply_probe_data);
+  R_RegisterCCallable("pomp","apply_probe_sim",(DL_FUNC) &apply_probe_sim);
+  R_RegisterCCallable("pomp","systematic_resampling",(DL_FUNC) &systematic_resampling);
+
   // Register routines
   R_registerRoutines(info,NULL,callMethods,NULL,NULL);
   R_useDynamicSymbols(info,TRUE);
