@@ -11,16 +11,17 @@
 SEXP do_rprocess (SEXP object, SEXP xstart, SEXP times, SEXP params, SEXP offset, SEXP gnsi)
 {
   int nprotect = 0;
-  int *xdim, nvars, npars, nreps, nrepsx, ntimes, off;
-  SEXP X, Xoff, copy, fn, fcall, rho;
+  int *xdim, type, nvars, npars, nreps, nrepsx, ntimes, off;
+  SEXP X, Xoff, copy, rproc, args, zeronames, tcovar, covar;
   SEXP dimXstart, dimP, dimX;
   const char *dimnm[3] = {"variable","rep","time"};
 
   PROTECT(gnsi = duplicate(gnsi)); nprotect++;
 
+  PROTECT(times = AS_NUMERIC(times)); nprotect++;
   ntimes = length(times);
   if (ntimes < 2) {
-    errorcall(R_NilValue,"in 'rprocess': length(times)<2: with no transitions, there is no work to do.");
+    errorcall(R_NilValue,"in 'rprocess': length(times) < 2: with no transitions, there is no work to do.");
   }
 
   off = *(INTEGER(AS_INTEGER(offset)));
@@ -35,7 +36,7 @@ SEXP do_rprocess (SEXP object, SEXP xstart, SEXP times, SEXP params, SEXP offset
   PROTECT(params = as_matrix(params)); nprotect++;
   PROTECT(dimP = GET_DIM(params)); nprotect++;
   xdim = INTEGER(dimP);
-  npars = xdim[0]; nreps = xdim[1]; 
+  npars = xdim[0]; nreps = xdim[1];
 
   if (nrepsx > nreps) {		// more ICs than parameters
     if (nrepsx % nreps != 0) {
@@ -51,9 +52,9 @@ SEXP do_rprocess (SEXP object, SEXP xstart, SEXP times, SEXP params, SEXP offset
       src = REAL(copy);
       tgt = REAL(params);
       for (j = 0; j < nrepsx; j++) {
-	for (k = 0; k < npars; k++, tgt++) {
-	  *tgt = src[k+npars*(j%nreps)];
-	}
+        for (k = 0; k < npars; k++, tgt++) {
+          *tgt = src[k+npars*(j%nreps)];
+        }
       }
     }
     nreps = nrepsx;
@@ -71,45 +72,60 @@ SEXP do_rprocess (SEXP object, SEXP xstart, SEXP times, SEXP params, SEXP offset
       src = REAL(copy);
       tgt = REAL(xstart);
       for (j = 0; j < nreps; j++) {
-	for (k = 0; k < nvars; k++, tgt++) {
-	  *tgt = src[k+nvars*(j%nrepsx)];
-	}
+        for (k = 0; k < nvars; k++, tgt++) {
+          *tgt = src[k+nvars*(j%nrepsx)];
+        }
       }
     }
   }
 
+  PROTECT(rproc = GET_SLOT(object,install("rprocess"))); nprotect++;
+  PROTECT(args = VectorToPairList(GET_SLOT(object,install("userdata")))); nprotect++;
+  PROTECT(zeronames = GET_SLOT(object,install("zeronames"))); nprotect++;
+  PROTECT(covar = GET_SLOT(object,install("covar"))); nprotect++;
+  PROTECT(tcovar = GET_SLOT(object,install("tcovar"))); nprotect++;
+
   // extract the process function
-  PROTECT(fn = GET_SLOT(object,install("rprocess"))); nprotect++;
-  // construct the call
-  PROTECT(fcall = VectorToPairList(GET_SLOT(object,install("userdata")))); nprotect++;
-  PROTECT(fcall = LCONS(gnsi,fcall)); nprotect++;
-  SET_TAG(fcall,install(".getnativesymbolinfo"));
-  PROTECT(fcall = LCONS(GET_SLOT(object,install("zeronames")),fcall)); nprotect++;
-  SET_TAG(fcall,install("zeronames"));
-  PROTECT(fcall = LCONS(GET_SLOT(object,install("covar")),fcall)); nprotect++;
-  SET_TAG(fcall,install("covar"));
-  PROTECT(fcall = LCONS(GET_SLOT(object,install("tcovar")),fcall)); nprotect++;
-  SET_TAG(fcall,install("tcovar"));
-  PROTECT(fcall = LCONS(params,fcall)); nprotect++;
-  SET_TAG(fcall,install("params"));
-  PROTECT(fcall = LCONS(AS_NUMERIC(times),fcall)); nprotect++;
-  SET_TAG(fcall,install("times"));
-  PROTECT(fcall = LCONS(xstart,fcall)); nprotect++;
-  SET_TAG(fcall,install("xstart"));
-  PROTECT(fcall = LCONS(fn,fcall)); nprotect++;
-  PROTECT(rho = (CLOENV(fn))); nprotect++; // environment of the function
-  PROTECT(X = eval(fcall,rho)); nprotect++; // do the call
+  type = *(INTEGER(GET_SLOT(rproc,install("type"))));
+  switch (type) {
+  case 1: // one-step simulator
+  {
+    SEXP fn;
+    double deltat = 1.0;
+    PROTECT(fn = GET_SLOT(rproc,install("step.fn"))); nprotect++;
+    PROTECT(X = euler_model_simulator(
+      fn,xstart,times,params,deltat,type,
+      zeronames,tcovar,covar,args,gnsi)); nprotect++;
+  }
+    break;
+  case 2: case 3: // discrete-time and Euler
+  {
+    SEXP fn;
+    double deltat;
+    PROTECT(fn = GET_SLOT(rproc,install("step.fn"))); nprotect++;
+    deltat = *(REAL(AS_NUMERIC(GET_SLOT(rproc,install("delta.t")))));
+    PROTECT(X = euler_model_simulator(fn,xstart,times,params,deltat,type,
+        zeronames,tcovar,covar,args,gnsi)); nprotect++;
+  }
+    break;
+  case 4: // Gillespie's method
+  {
+    SEXP fn, vmatrix, hmax;
+    PROTECT(fn = GET_SLOT(rproc,install("rate.fn"))); nprotect++;
+    PROTECT(vmatrix = GET_SLOT(rproc,install("v"))); nprotect++;
+    PROTECT(hmax = GET_SLOT(rproc,install("hmax"))); nprotect++;
+    PROTECT(X = SSA_simulator(fn,xstart,times,params,vmatrix,tcovar,covar,
+        zeronames,hmax,args,gnsi)); nprotect++;
+  }
+    break;
+  case 0: default:
+    errorcall(R_NilValue,"'rprocess' is undefined.");
+    break;
+  }
+
   PROTECT(dimX = GET_DIM(X)); nprotect++;
-  if ((isNull(dimX)) || (length(dimX) != 3)) {
-    errorcall(R_NilValue,"in 'rprocess': user 'rprocess' must return a rank-3 array");
-  }
   xdim = INTEGER(dimX);
-  if ((xdim[0] != nvars) || (xdim[1] != nreps) || (xdim[2] != ntimes)) {
-    errorcall(R_NilValue,"in 'rprocess': user 'rprocess' must return a %d x %d x %d array",nvars,nreps,ntimes);
-  }
-  if (isNull(GET_ROWNAMES(GET_DIMNAMES(X)))) {
-    errorcall(R_NilValue,"in 'rprocess': user 'rprocess' must return an array with rownames");
-  }
+
   if (off > 0) {
     xdim[2] -= off;
     PROTECT(Xoff = makearray(3,xdim)); nprotect++;
